@@ -1,8 +1,7 @@
-# -*- coding: utf-8 -*-
 import os
 import sys
 import runcmd
-
+import yaml
 curPath = os.path.abspath(os.path.dirname(__file__))
 rootPath = os.path.split(curPath)[0]
 sys.path.append(rootPath)
@@ -15,6 +14,7 @@ import re
 from urllib.parse import urlparse
 from prettytable import PrettyTable, ALL
 import pkg_resources  # part of setuptools
+from progress.bar import Bar
 
 
 # yaml 数据模型
@@ -54,23 +54,6 @@ class YamlModuleModel:
         self.inhibit_warnings = inhibit_warnings
 
 
-# 输出合并结果，如果result = 1 表示成功，result = 0 表示失败
-class YamlBranchModel:
-    module = ''
-    pod = ''
-    result = 0
-    pod = ''
-
-    def __init__(self, module, res, pod=''):
-        """
-        :param module: 模块名字
-        :param res: 结果 0 表示失败 1 表示成功
-        """
-        self.module = module
-        self.result = res
-        self.pod = pod
-
-
 # 文件状态，如果result = 1 表示成功，result = 0 表示失败 -1 表示其他异常
 class ModuleStatusModel:
     module = ''
@@ -413,9 +396,12 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
     # 获取ymal 数据
     podfile_module_data = yaml_data(yaml_path)
     dependenceList = podfile_module_data["dependencies"]
-    before_branch = podfile_module_data["branch"]
+    before_branch = ''
+    if "branch" in podfile_module_data:
+        before_branch = podfile_module_data['branch']
     # 转换成模型数组
     conver_deplist = load_yaml(dependenceList)
+    print("origin=======" + str(dependenceList))
 
     index = 0
     for a in conver_deplist:
@@ -426,6 +412,21 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
                                "inhibit_warnings": a.inhibit_warnings}
                 dependenceList[index] = module_dict
         index += 1
+    print("convert=======" + str(dependenceList))
+    except_list = [
+        module for module in branch_result if module.module not in [
+            pod.module for pod in conver_deplist]]
+    if len(except_list) > 0:
+        for mo_re in except_list:
+            module_dict = {
+                "module": mo_re.module,
+                "pod": a.pod,
+                "git": a.git,
+                "branch": n_branch
+            }
+            dependenceList.append(module_dict)
+
+    print("except_list=======" + str(dependenceList))
     if not (before_branch and len(before_branch) > 0):
         before_branch = n_branch
     podfile_data = {"version": "1.0.0", "branch": str(before_branch), "dependencies": dependenceList}
@@ -441,9 +442,17 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
     else:
         # 获取ymal 数据
         podfile_module_data = yaml_data(local_yaml_path)
-        local_dependenceList = podfile_module_data["dependencies"]
-        # 转换成模型数组
-        conver_deplist = load_yaml(local_dependenceList)
+        local_dependenceList = []
+        conver_deplist = []
+        if podfile_module_data:
+
+            if "dependencies" in podfile_module_data:
+                local_dependenceList = podfile_module_data["dependencies"]
+            if not local_dependenceList:
+                local_dependenceList = []
+            # 转换成模型数组
+            conver_deplist = load_yaml(local_dependenceList)
+
         for yamlModel in branch_result:
             contain = False
             for cm in conver_deplist:
@@ -465,9 +474,8 @@ def update_module_files(yaml_path, local_yaml_path, branch_result, n_branch, mod
         for a in conver_deplist:
             module_dict = {"module": a.module, "pod": a.pod, "path": modules_name + "/" + a.module}
             after_convert.append(module_dict)
-
     local_module_data = {"version": "1.0.0", "branch": str(n_branch), "dependencies": after_convert}
-    update_yaml(local_yaml_path, local_module_data)
+    save_dict_to_yaml(local_module_data, local_yaml_path)
 
 
 # 更新podfileModule文件
@@ -593,10 +601,11 @@ def create_branch(module_list, module_f_path, n_branch, modules, c_path):
     :param c_path: 当前的工作目录
     :return:
     """
-    index = 0
     create_branch_result = []
+    bar = Bar('install...', fill="*", max=len(modules))
     for a in module_list:
         if a.module in modules:
+            bar.next()
             if (a.branch and len(str(a.branch)) > 0) or (a.tag and len(str(a.tag)) > 0):
                 filename = a.module
                 module_path = module_f_path + filename + "/"
@@ -616,8 +625,8 @@ def create_branch(module_list, module_f_path, n_branch, modules, c_path):
                 new_branch_model = ModuleStatusModel(a.module, a.pod, 0, n_branch,
                                                      "podfileModule.yaml 中组件: " + a.pod + " 的branch或者tag为空, 不能确定要拉取的代码的位置")
                 create_branch_result.append(new_branch_model)
-        index += 1
         os.chdir(c_path)
+    bar.finish()
     return create_branch_result
 
 
@@ -656,13 +665,13 @@ def new_branch(filepath, git, f_branch, f_tag, n_branch):
     :param n_branch: 新分支名
     :return: 成功返回新分支名，失败返回空字符串
     """
-    pull_branch = f_branch
+    origin_branch = f_branch
     if not (f_branch and len(f_branch) > 0):
-        pull_branch = "master"
+        origin_branch = "master"
     if f_branch and len(f_branch) > 0 and f_branch == n_branch:
-        pull_branch = "master"
+        origin_branch = "master"
     # clone 代码
-    git_clone_command = "git clone -b " + pull_branch + " " + git + " " + filepath
+    git_clone_command = "git clone -b " + origin_branch + " " + git + " " + filepath
     # 创建一个新分支
     git_create_branch = "git branch " + n_branch
     if f_tag and len(f_tag) > 0:
@@ -694,7 +703,7 @@ def new_branch(filepath, git, f_branch, f_tag, n_branch):
 
 
 # 基于PodfileLocal，提交所有模块开发分支代码
-def commit_branch(module_list, module_f_path, msg):
+def commit_branch(module_list, include_list, module_f_path, msg):
     """
     基于列表，提交对应开发分支代码
     :param module_list: 模块列表
@@ -703,7 +712,12 @@ def commit_branch(module_list, module_f_path, msg):
     :return: 返回操作的分支
     """
     commit_result = []
-    for module in module_list:
+    invert_list = module_list
+    if len(include_list) > 0:
+        invert_list = [i for i in module_list if i.module in include_list]
+    bar = Bar('commit...', fill="*", max=len(invert_list))
+    for module in invert_list:
+        bar.next()
         filename = module.module
         module_path = module_f_path + filename + "/"
         git = RepoGit(module_path)
@@ -722,21 +736,25 @@ def commit_branch(module_list, module_f_path, msg):
                 error_msg = "提交失败"
         modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, error_msg)
         commit_result.append(modul_branch_model)
+    bar.finish()
     return commit_result
 
 
 # 基于podfileModule，提交模块开发分支代码
-def pull_branch(module_list, module_f_path):
+def pull_branch(module_list, include_list, module_f_path):
     """
     基于列表，提交对应开发分支代码
     :param module_list: 模块列表
     :param module_f_path: 路径
     :return: 返回操作的分支
     """
-
-    index = 0
+    invert_list = module_list;
+    if len(include_list) > 0:
+        invert_list = [i for i in module_list if i.module in include_list]
     pull_result = []
-    for module in module_list:
+    bar = Bar('pull...', fill="*", max=len(invert_list))
+    for module in invert_list:
+        bar.next()
         filename = module.module
         module_path = module_f_path + filename + "/"
         git = RepoGit(module_path)
@@ -756,11 +774,12 @@ def pull_branch(module_list, module_f_path):
 
         modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, msg)
         pull_result.append(modul_branch_model)
+    bar.finish()
     return pull_result
 
 
 # 基于podfileModule，提交模块开发分支代码
-def push_branch(module_list, module_f_path):
+def push_branch(module_list, include_list, module_f_path):
     """
     基于列表，提交对应开发分支代码
     :param module_list: 模块列表
@@ -770,7 +789,12 @@ def push_branch(module_list, module_f_path):
 
     index = 0
     pull_result = []
-    for module in module_list:
+    invert_list = module_list;
+    if len(include_list) > 0:
+        invert_list = [i for i in module_list if i.module in include_list]
+    bar = Bar('push...', fill="*", max=len(invert_list))
+    for module in invert_list:
+        bar.next()
         filename = module.module
         module_path = module_f_path + filename + "/"
         git = RepoGit(module_path)
@@ -782,6 +806,7 @@ def push_branch(module_list, module_f_path):
             msg = "本地有变动未提交，请确认"
         else:
             try:
+                git.repo.git.pull('--progress', '--no-rebase', 'origin', branch)
                 git.repo.git.push('--progress', 'origin', branch)
             except Exception as e:
                 result = 0
@@ -789,6 +814,7 @@ def push_branch(module_list, module_f_path):
 
         modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, msg)
         pull_result.append(modul_branch_model)
+    bar.finish()
     return pull_result
 
 
@@ -804,8 +830,60 @@ def merge_branch(module_list, convert_list, module_f_path, from_branch):
     """
     index = 0
     pull_result = []
+    invert_list = convert_list
+    if len(module_list) > 0:
+        invert_list = [i for i in convert_list if i.module in module_list]
+    bar = Bar('merge...', fill="*", max=len(invert_list))
+    for module in invert_list:
+        if module.module in module_list:
+            bar.next()
+            filename = module.module
+            module_path = module_f_path + filename + "/"
+            git = RepoGit(module_path)
+            branch = git.getCurrentBranch()
+            msg = ''
+            result = 1
+            if git.is_dirty():
+                result = -1
+                msg = "本地有变动未提交，请确认"
+            else:
+                branchs = git.get_branches()
+                if from_branch not in branchs:
+                    result = -1
+                    msg = "分支： {0} 不存在".format(from_branch)
+                else:
+                    try:
+                        if 'origin/' not in from_branch:
+                            from_branch = "origin/" + from_branch
+                        git.repo.git.pull('--progress', '--no-rebase', 'origin', branch)
+                        git.repo.git.merge(from_branch)
+                        git.repo.git.push('--progress', 'origin', branch)
+                    except Exception as e:
+                        result = 0
+                        msg = str(e)
+            modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, msg)
+            pull_result.append(modul_branch_model)
+        index += 1
+    bar.finish()
+    return pull_result
+
+
+# 合并from_branch中代码到当前的开发分支
+def rebase_branch(module_list, convert_list, module_f_path, from_branch):
+    """
+    基于列表，提交对应开发分支代码
+    :param module_list: 模块列表
+    :param convert_list: 模块列表
+    :param module_f_path: 路径
+    :param from_branch: 要合并的分支
+    :return: 返回操作的分支
+    """
+    index = 0
+    pull_result = []
+    bar = Bar('rebase...', fill="*", max=len(module_list))
     for module in convert_list:
         if module.module in module_list:
+            bar.next()
             filename = module.module
             module_path = module_f_path + filename + "/"
             git = RepoGit(module_path)
@@ -830,7 +908,7 @@ def merge_branch(module_list, convert_list, module_f_path, from_branch):
             modul_branch_model = ModuleStatusModel(module.module, module.pod, result, branch, msg)
             pull_result.append(modul_branch_model)
         index += 1
-
+    bar.finish()
     return pull_result
 
 
@@ -847,7 +925,9 @@ def release_branch(module_list, include_list, tag_path, c_path, f_branch):
 
     index = 0
     merge_result = []
+    bar = Bar('release...', fill="*", max=len(module_list))
     for a in module_list:
+        bar.next()
         if not (a.branch and len(a.branch) > 0) and not (a.tag and len(a.tag) > 0):
             a.branch = f_branch
         if a.branch and len(a.branch) > 0:
@@ -864,6 +944,7 @@ def release_branch(module_list, include_list, tag_path, c_path, f_branch):
 
         index += 1
         os.chdir(c_path)
+    bar.finish()
     return merge_result
 
 
@@ -889,6 +970,12 @@ def printTable(list, info):
         debugInfo("\n[!] 暂无 \"{0}\"\n".format(info))
 
 
+def save_dict_to_yaml(dict_value: dict, save_path: str):
+    """dict保存为yaml"""
+    with open(save_path, 'w') as file:
+        file.write(yaml.dump(dict_value, sort_keys=False))
+
+
 # 创建分支的对象
 # branch 组件新分支名字
 class yhgit:
@@ -923,6 +1010,77 @@ class yhgit:
     #     self.tag = tag
 
     # def __init__(self):
+    def init(self, n_branch, git):
+        """
+        如果本地没有PodfileModule，该命令会自动生成PodfileModule文件，然后可以使用后续指令进行状态检测，推拉代码，提交及打tag的操作，如果有就会只执行后续的新增分支及拉取代码的操作
+        :return:
+        """
+        c_path = ""
+        f_path = os.getcwd()
+        file_name = os.path.basename(git)
+        file_name, file_ext = os.path.splitext(file_name)
+        modules_name = "modules"
+        modules_path = c_path + "modules/"
+        module_path = modules_path + file_name
+        result = auto_create_branch(module_path, git, n_branch, '', n_branch)
+        os.chdir(f_path)
+        # 子目录，用来存放子模块仓库, 如果没有需要创建一个
+        if not os.path.exists(modules_path):
+            create_file(modules_path)
+        error = ""
+        res = 1
+        if result and len(result) > 0:
+            # 新分支创建成功
+            # 更新模块的名字{file_name}为{podspec}名字
+            newfile_name = ""
+            files = [filename for filename in os.listdir(f_path + "/" + module_path) if filename.endswith(".podspec")]
+            if len(files) > 0:
+                newfile_name = files[0]
+                newfile_name, file_ext = os.path.splitext(newfile_name)
+
+            newfile_path = modules_path + newfile_name
+            # 判断是否有newfile_name对应的文件夹
+            find = False
+            for name in os.listdir(f_path + "/" + modules_path):
+                # 检查文件夹名称是否以指定字符串开头
+                if name.startswith(newfile_name) and os.path.isdir(os.path.join(f_path + "/" + modules_path, name)):
+                    find = True
+            if not find and newfile_name and len(newfile_name) > 0:
+                os.rename(f_path + "/" + module_path, f_path + "/" + newfile_path)
+            else:
+                # 删除旧的文件夹
+                if os.path.exists(f_path + "/" + module_path):
+                    shutil.rmtree(f_path + "/" + module_path)
+                res = 1
+                error = "存在同名文件夹，新建分支失败，请手动切换"
+        # 判断是否有PodfileModule文件夹
+        localyamlPath = c_path + 'PodfileLocal.yaml'
+        yamlPath = c_path + 'PodfileModule.yaml'
+        if not os.path.getsize(localyamlPath):
+            shutil.rmtree(localyamlPath)
+
+        if not os.path.getsize(yamlPath):
+            shutil.rmtree(yamlPath)
+
+        if not os.path.exists(yamlPath):
+            open(yamlPath, 'w').close()
+            after_convert = []
+            module_dict = {"module": newfile_name, "pod": newfile_name, "git": git, "branch": n_branch}
+            after_convert.append(module_dict)
+            podmodule_data = {"version": "1.0.0", "branch": str(n_branch), "dependencies": after_convert}
+            save_dict_to_yaml(podmodule_data, yamlPath)
+
+        branch_res = [ModuleStatusModel(newfile_name, newfile_name, res, n_branch, error)]
+        update_module_files(yamlPath, localyamlPath, branch_res, n_branch, modules_name)
+        succ_list = []
+        fail_list = []
+        for merg_Model in branch_res:
+            if merg_Model.result == 1:
+                succ_list.append(merg_Model)
+            else:
+                fail_list.append(merg_Model)
+        printTable(succ_list, "分支创建成功的模块")
+        printTable(fail_list, "分支创建失败的模块")
 
     # 初始化项目仓库，自动拉取组件仓库，并创建新分支
     def install(self, n_branch, modules=[]):
@@ -970,7 +1128,7 @@ class yhgit:
             printTable(fail_list, "分支创建失败的模块")
 
     # 获取podfileLocal.yaml中各模块状态, 提示： 1. 很干净没有未提交的代码 2. 有改动未提交 3. 没有相关文件报错误
-    def status(self):
+    def status(self, include_modules):
         """
         获取podfileLocal.yaml中各模块状态, 提示： 1. 很干净没有未提交的代码 2. 有改动未提交
         1. 如果podfileLocal.yaml是空则终止
@@ -993,7 +1151,12 @@ class yhgit:
         # 转换成模型数组
         module_list = load_yaml(local_dependenceList)
         status_result = []
-        for module in module_list:
+        invert_list = module_list
+        if len(include_modules) > 0:
+            invert_list = [i for i in module_list if i.module in include_modules]
+        bar = Bar('status...', fill="*", max=len(invert_list))
+        for module in invert_list:
+            bar.next()
             result = 0
             branchname = ''
             msg = ''
@@ -1010,10 +1173,10 @@ class yhgit:
                         msg = repogit.getStatusFormatStr
                 else:
                     result = -1
-                    msg = "请检查模块：" + module.module + " 的路径 " + module.path + " 是否为空";
+                    msg = "请检查模块：" + module.module + " 的路径 " + module.path + " 是否为空"
             new_branch_model = ModuleStatusModel(module.module, module.pod, result, branchname, msg)
             status_result.append(new_branch_model)
-
+        bar.finish()
         if len(status_result) == 0:
             debugInfo("没有要执行git status的模块")
         else:
@@ -1032,7 +1195,7 @@ class yhgit:
             printTable(error_list, "检查失败的模块")
 
     # 自动提交本地修改的内容
-    def commit(self, msg):
+    def commit(self, include_list, msg):
         """
         基于podfileModules的配置信息，提交本地开发分支代码
         :param msg: 这些模块不需要修改，不能提交代码
@@ -1059,7 +1222,7 @@ class yhgit:
         local_dependenceList = podfile_module_data["dependencies"]
         # 转换成模型数组
         module_list = load_yaml(local_dependenceList)
-        branch_res = commit_branch(module_list, modules_path, msg)
+        branch_res = commit_branch(module_list, include_list, modules_path, msg)
         if len(branch_res) == 0:
             debugInfo("没有要提交代码的模块")
         else:
@@ -1074,7 +1237,7 @@ class yhgit:
             printTable(fail_list, "代码提交失败的模块")
 
     # 自动拉取远端代码
-    def pull(self):
+    def pull(self, include_list):
         """
         基于PodfileLocal的配置信息，提交本地开发分支代码
         :return:
@@ -1098,7 +1261,7 @@ class yhgit:
         local_dependenceList = podfile_module_data["dependencies"]
         # 转换成模型数组
         module_list = load_yaml(local_dependenceList)
-        branch_res = pull_branch(module_list, modules_path)
+        branch_res = pull_branch(module_list, include_list, modules_path)
         if len(branch_res) == 0:
             logging.info("没有要拉取远端代码的模块")
         else:
@@ -1113,7 +1276,7 @@ class yhgit:
             printTable(fail_list, "代码拉取失败的模块")
 
     # 自动推送本地修改到远端
-    def push(self):
+    def push(self, include_list=[]):
         """
         基于PodfileLocal的配置信息，拉取本地开发分支代码
         :return:
@@ -1137,7 +1300,7 @@ class yhgit:
         local_dependenceList = podfile_module_data["dependencies"]
         # 转换成模型数组
         module_list = load_yaml(local_dependenceList)
-        branch_res = push_branch(module_list, modules_path)
+        branch_res = push_branch(module_list, include_list, modules_path)
         if len(branch_res) == 0:
             logging.info("没有要推送代码的模块")
         else:
@@ -1287,13 +1450,28 @@ def main(argvs=None):
             if os.path.exists(local_yaml_path):
                 podfile_module_data = yaml_data(local_yaml_path)
                 branchname = podfile_module_data.get("branch", None)
-        print(branchname)
         if not (branchname and len(branchname) > 0):
             logging.error("请指定分支名字，\n 1. 指令后面指定分支 \n2. 在PodfileLoacal.yaml中指定branch")
         if not (modules and len(modules) > 0):
             logging.error("请指定具体的模块")
         else:
             yhgit().install(branchname, modules)
+    elif command == "init":
+        git = ""
+        if len(argvs) > 2:
+            subcommand = argvs[2]
+            if subcommand and len(subcommand) > 0 and subcommand == "-b" and len(argvs) >= 4:
+                # 需要获取具体的分支名
+                branchname = argvs[3]
+                if branchname and len(branchname) > 0 and len(argvs) >= 5:
+                    # 获取需要创建分支的组件
+                    git = argvs[4]
+
+        if (not (git and len(git) > 0)) or (not (branchname and len(branchname) > 0)):
+            logging.error("请指定分支或者git地址")
+        else:
+            yhgit().init(branchname, git)
+
     elif command == "merge":
         if len(argvs) > 2:
             subcommand = argvs[2]
@@ -1313,7 +1491,10 @@ def main(argvs=None):
 
     elif command == "status":
         # 查看本地代码的状态
-        yhgit().status()
+        modules = []
+        if len(argvs) > 2:
+            modules = argvs[2:]
+        yhgit().status(modules)
     elif command == "commit":
         commit_msg = ''
         if len(argvs) > 2:
@@ -1321,15 +1502,23 @@ def main(argvs=None):
             if subcommand and len(subcommand) > 0 and subcommand == "-m":
                 # 需要获取具体的分支名
                 commit_msg = argvs[3]
+                if commit_msg and len(commit_msg) > 0 and len(argvs) > 3:
+                    modules = argvs[4:]
         if not (commit_msg and len(commit_msg) > 0):
             debugInfo("必须添加-m 和 commit_msg")
         else:
-            yhgit().commit(commit_msg)
+            yhgit().commit(modules, commit_msg)
     elif command == "pull":
-        yhgit().pull()
+        modules = []
+        if len(argvs) > 2:
+            modules = argvs[2:]
+        yhgit().pull(modules)
 
     elif command == "push":
-        yhgit().push()
+        modules = []
+        if len(argvs) > 2:
+            modules = argvs[2:]
+        yhgit().push(modules)
 
     elif command == "release":
         modules = []
@@ -1344,6 +1533,10 @@ def main(argvs=None):
     elif command == "help" or command == "-h":
 
         table = PrettyTable(["编号", "指令", "参数", "说明", "使用"])
+        table.add_row(
+            [0, "init", "-b 指定分支名; 组件git地址",
+             "根据git地址，新建分支，然后自动更新PodfileModule.yaml及PodfileLocal.yaml文件",
+             "yhgit init -b test git@gitlab.yonghui.cn:operation-xm-qdjg/yhfoundation-ios.git"])
         table.add_row(
             [0, "install", "-b 指定分支名; 组件 指定 新建分支的组件",
              "根据分支branch，然后基于组件或者PodfileLocal.yaml中的配置，新建分支branch",
@@ -1392,7 +1585,7 @@ if __name__ == '__main__':
 
     """
     argv[0]:  获取命令的类型 如下
-    install  
+    install
 
     argv[1]:  命令的
 
